function uberTrackerSort(%datamap, %method) {

	if (%method $= "") %method = "killsCompare";
	$UberTrackerMap = %datamap;
	quickSort(%datamap.keys,%method);
	$UberTrackerMap = "";
}

function killsCompare(%a, %b) {

	if ($UberTrackerMap.value(%a).kills > $UberTrackerMap.value(%b).kills) return -1;
	else if ($UberTrackerMap.value(%a).kills < $UberTrackerMap.value(%b).kills) return 1;
	else return 0;
}

function deathsCompare(%a, %b) {

	if ($UberTrackerMap.value(%a).deaths > $UberTrackerMap.value(%b).deaths) return -1;
	else if ($UberTrackerMap.value(%a).deaths < $UberTrackerMap.value(%b).deaths) return 1;
	else return 0;
}

function valueCompare(%a, %b) {

	if ($UberTrackerMap.value(%a) > $UberTrackerMap.value(%b)) return -1;
	else if ($UberTrackerMap.value(%a) < $UberTrackerMap.value(%b)) return 1;
	else return 0;
}

function nCompare(%a, %b) {
	if (%a > %b) return  1;
	if (%a < %b) return -1;
	else return 0;
}

function insertionSort(%vector, %compareFunc, %begin, %end) {

	if ((%vector $= "") || (!isObject(%vector)) || (%vector.class !$= Vector)) {
		error("Can only sort data in Vector class objects.");
		return;
	}

	if (%compareFunc $= "") %compareFunc = strcmp;
	if (%begin $= "") %begin = 0;
	if (%end $= "") %end = %vector.lastIndex;

	for (%i = (%begin+1); %i <= %end; %i++) {
		%temp = %vector.array[%i];
		for (%j = (%i-1); %j >= %begin; %j--) {
			if (call(%compareFunc, %vector.array[%j],%temp) > 0) {
				%vector.array[%j+1] = %vector.array[%j];
			}
			else {
				break;
			}
		}
		%vector.array[%j+1] = %temp;
	}
}

function qSort(%vector, %compareFunc, %L, %R) {

	if ((%vector $= "") || (!isObject(%vector)) || (%vector.class !$= Vector)) {
		error("Can only sort data in Vector class objects.");
		return;
	}

	if (%compareFunc $= "") %compareFunc = strcmp;
	if (%L $= "") %L = 0;
	if (%R $= "") %R = %vector.lastIndex;

	while ((%R - %L) >= 9) {
		%diff = %R-%L;
		%mid = getRandom(%L+1,%R-1);
		if (call(%compareFunc, %vector.array[%L], %vector.array[%mid]) > 0) {
			%temp = %vector.array[%L];
			%vector.array[%L] = %vector.array[%mid];
			%vector.array[%mid] = %temp;
		}
		if (call(%compareFunc, %vector.array[%L], %vector.array[%R]) > 0) {
			%temp = %vector.array[%L];
			%vector.array[%L] = %vector.array[%R];
			%vector.array[%R] = %temp;
		}
		if (call(%compareFunc, %vector.array[%mid], %vector.array[%R]) > 0) {
			%temp = %vector.array[%mid];
			%vector.array[%mid] = %vector.array[%R];
			%vector.array[%R] = %temp;
		}
		%temp = %vector.array[%mid];
		%vector.array[%mid] = %vector.array[%R-1];
		%vector.array[%R-1] = %temp;

		%i = %L;
		%j = %R-1;
		%ppivot = %vector.array[%j];

		while(true) {
			while (call(%compareFunc, %vector.array[%i++], %ppivot) < 0) {}
			while (call(%compareFunc, %vector.array[%j--], %ppivot) > 0) {}
			if (%i >= %j) break;
			%temp = %vector.array[i];
			%vector.array[i] = %vector.array[j];
			%vector.array[j] = %temp;
		}

		%temp = %vector.array[%R-1];
		%vector.array[%R-1] = %vector.array[%i];
		%vector.array[%i] = %temp;
		if (%i - %L > %R - %i) {
			qSort(%vector, %compareFunc, %i+1, %R);
			%R = %i - 1;
		}
		else {
			qSort(%vector, %compareFunc, %L, %i-1);
			%L = %i + 1;
		}
	}
}

function quickSort(%vector, %compareFunc, %first, %last) {

	if ((%vector $= "") || (!isObject(%vector)) || (%vector.class !$= Vector)) {
		error("Can only sort data in Vector class objects.");
		return;
	}

	if (%compareFunc $= "") %compareFunc = strcmp;
	if (%first $= "") %first = 0;
	if (%last $= "") %last = %vector.lastIndex;

	qSort(%vector,%compareFunc,%first,%last);
	insertionSort(%vector,%compareFunc,%first,%last);
}

function test(%size) {

	%size = mFloor(%size);

	%X = Container::newVector();
	echo("Making Samples...");
	%str = "";

	for (%i=0;%i<%size;%i++) {
		%X.pushBack(getRandom(-%size,%size));
	}

	echo("Sorting...");
	%time = getRealTime();
	quickSort(%X,nCompare);
	echo(getRealTime() - %time);

	echo("Sorting...");
	%time = getRealTime();
	quickSort(%X,nCompare);
	echo(getRealTime() - %time);

	//for (%i=0;%i<%size;%i++) {
	//	%str = %str @ %X.valueAt(%i) @ " ";
	//}
	//echo(%str);
	%X.delete();
}