// #autoload
// #name = UberTracker
// #version = 2.0.4
// #date = October 12, 2002
// #author = Paul Tousignant
// #warrior = UberGuy (FT)
// #email = uberguy@tribalwar.com
// #web = http://scripts.tribalwar.com/uberguy
// #description = Detailed history logging of who you've killed or been killed by in game.
// #category = UberGuy's Scripts
// #status = Beta
// #config = UberTrackerGUI
// #credit = PanamaJack - for the kill tracker that inspired me to make this one. ;) Also my debrief GUI display is heavily based on his.
// #credit = FrostyCubes - my score HUD is a modified version of his.
// #credit = Astro-Lorf - for feedback on what CTF stats.
// #credit = Avox - bug finder extraordinaire.
// #readme = scripts/UberGuy/uberTracker.txt
// #include = support/map.cs 1.0.9
// #include = support/events.cs 1.0.4
// #include = support/flag_tracker.cs 0.0.3
// #include = support/docking_tools.cs
// #include = support/kill_callbacks.cs
// #include = support/mission_callbacks.cs

exec("scripts/UberGuy/quicksort.cs");

function uberTracker_createTrackerMap(%name) {

	%map = new ScriptObject(%name @ "_trackerMap") {

		playerData = Container::newVectorMap();
		weaponData = Container::newVectorMap();
		assetsData  = Container::newVectorMap();

		ctrlK = 0;
		suicides = 0;

		kills = 0;
		deaths = 0;

		TK_kills = 0;
		TK_deaths = 0;

		killers = 0;
		victims = 0;

		TK_killers = 0;
		TK_victims = 0;
	};

	return %map;
}

function uberTracker::createTrackerInfo(%this, %name) {

	%info = new ScriptObject(%name @ "_TrackerData") {

		kills = 0;
		deaths = 0;
		TK_kills = 0;
		TK_deaths = 0;
	};
	return %info;
}

// Code to handle asset destruction events
// Message callbacks and lookup tables defined at bottom of script
//=============================================================================
function uberTracker_handleDestroy(%msgType, %msgString, %a0, %a1) {

	%msgType = detag(%msgType);
	if (%msgType $= "msgVehicleScore") %type = detag(%a1);
	else {
		%type = uberTracker.statType[%msgType];
	}
	%name = uberTracker.statName[%type];

	if (%name $= "") %name = %type;

	if (!uberTracker.mapData.assetsData.hasKey(%type)) uberTracker.mapData.assetsData.add(%type,1);
	else uberTracker.mapData.assetsData.incrementValue(%type);

	if (!uberTracker.logData.assetsData.hasKey(%type)) uberTracker.logData.assetsData.add(%type,1);
	else uberTracker.logData.assetsData.incrementValue(%type);

	if ($uberPref::UT::popupAssets)
		uberTracker.display("<just:center><color:22ff22><spush><font:Univers Bold:16>You destroyed an enemy<spop>"
		 NL "<color:ffffff>" @ %name,2);
}

// CTF flag event code
//=============================================================================
function uberTracker::onCTFGrab(%this, %flagRef) {

	if (%flagRef.teamID == teamTracker.enemyTeamID) {
		if (%flagRef.actorCurrent == teamTracker.getSelfRef()) {
			%this.ctfAssist = false;
			%this.ctfMadeGrab = true;
			%this.mapData.ctfGrabCount++;
			%this.logData.ctfGrabCount++;

			%this.grabTime = getSimTime();

			%speed = mFormatFloat(getControlObjectspeed() * 3.6,"%.f");
			if (%speed > %this.mapData.ctfBestGrabSpeed) %this.mapData.ctfBestGrabSpeed = %speed;
			if (%speed > %this.logData.ctfBestGrabSpeed) %this.logData.ctfBestGrabSpeed = %speed;
		}
	}
}

function uberTracker::onCTFPicked(%this, %flagRef) {

	if (%flagRef.teamID == teamTracker.enemyTeamID) {
		if (%flagRef.actorCurrent == teamTracker.getSelfRef()) {
			if (!%this.ctfMadeGrab) %this.ctfAssist = true;
			%this.mapData.ctfPickCount++;
			%this.logData.ctfPickCount++;
		}
		else {
			if (%this.ctfMadeGrab) %this.ctfAssist = true;
		}
	}
}

function uberTracker::onCTFCap(%this, %flagRef) {

	if (%flagRef.teamID == teamTracker.enemyTeamID) {

		// I made a grab and no one else touched the flag before I capped it
		// Even if I dropped it I picked it up and capped
		if (%this.ctfMadeGrab && !%this.ctfAssist) {
			%this.mapData.ctfCleanCapCount++;
			%this.logData.ctfCleanCapCount++;

			%time = getSimTime()- %this.grabTime;
			error(%time);

			if (%this.mapData.ctfBestCapTime == 0) %this.mapData.ctfBestCapTime = 999999;
			if (%time < %this.mapData.ctfBestCapTime) %this.mapData.ctfBestCapTime = %time;

			if (%this.logData.ctfBestCapTime == 0) %this.logData.ctfBestCapTime = 999999;
			if (%time < %this.logData.ctfBestCapTime) %this.logData.ctfBestCapTime = %time;
		}

		// If other teammates touched it, so it was an assist on my part
		if (%this.ctfAssist) {
			%this.mapdata.ctfAssistCount++;
			%this.logdata.ctfAssistCount++;
		}

		// If I actully capped it...
		if (%flagRef.actorCurrent == teamTracker.getSelfRef()) {
			%this.mapData.ctfCapCount++;
			%this.logData.ctfCapCount++;

			// Record if it was an assist
			if (%this.ctfAssist) {
				%this.mapData.ctfCapOnAssistCount++;
				%this.logData.ctfCapOnAssistCount++;
			}
		}
		// If someone else capped it...
		else {
			// And I assisted...
			if (%this.ctfAssist) {
				// If I made the initial grab, credit that...
				if (%this.ctfMadeGrab) {
					%this.mapData.ctfGrabCausedCapCount++;
					%this.logData.ctfGrabCausedCapCount++;
				}
				// Else credit that I aided another capper
				else {
					%this.mapData.ctfAssistOtherCapperCount++;
					%this.logData.ctfAssistOtherCapperCount++;
				}
			}
		}
		%this.ctfAssist = %this.ctfMadeGrab = false;
	}
}

function uberTracker::onCTFReturn(%this, %flagRef) {

	if (%flagRef.teamID == teamTracker.enemyTeamID) {
		%this.ctfAssist = %this.ctfMadeGrab = false;
	}
	else {
		if (%flagRef.actorCurrent ==  teamTracker.getSelfRef()) {
			%this.mapData.ctfReturnCount++;
			%this.logData.ctfReturnCount++;
		}
	}
}

function uberTracker::onCTFKill(%this, %type, %killerRef, %victimRef, %weapon, %i_die, %i_win) {

	if (%i_die) {
		%this.mapData.ctfDropCount++;
		%this.logData.ctfDropCount++;
	}
	else if (%i_win) {
		%this.mapData.ctfCarrierKillCount++;
		%this.logData.ctfCarrierKillCount++;
	}
}

// CTF flag event popups
//=============================================================================

function uberTracker::ctfReturnPop(%this, %flagRef) {

	if (!$uberPref::UT::popupCTF) return;

	if (!isObject(%flagRef.actorCurrent)) {
		if (%flagRef.teamID == teamTracker.friendlyTeamID) {
			%this.display("<just:center><color:eeeeee><font:Univers:16>Your flag automatically\nreturned",2);
		}
		else {
			%this.display("<just:center><color:eeeeee><font:Univers:16>The<spush><color:ff9999>" SPC
				teamTracker.teamName[%flagRef.teamID] NL "<spop>flag automatically returned",2);
		}
	}
	else {
		if (teamTracker.getSelfRef() == %flagRef.actorCurrent) %name = "You";
		else %name = strippedName(%flagRef.actorCurrent);

		if (%flagRef.teamID == teamTracker.friendlyTeamID) {
			%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
				"<color:22ff22>returned your flag",2);
		}
		else {
			%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
				"<color:ff9999>returned the" SPC teamTracker.teamName[%flagRef.teamID] NL
				"flag",3);
		}
	}
}

function uberTracker::ctfPickupPop(%this, %flagRef) {

	if (!$uberPref::UT::popupCTF) return;

	if (teamTracker.getSelfRef() == %flagRef.actorCurrent) %name = "You";
	else %name = strippedName(%flagRef.actorCurrent);

	if (%flagRef.teamID == teamTracker.friendlyTeamID) {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:ff9999>grabbed your flag\nfrom the field",2);
	}
	else {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:22ff22>grabbed the<spush><color:ff9999>" SPC teamTracker.teamName[%flagRef.teamID] NL
			 "<spop>flag from the field",3);
	}
}

function uberTracker::ctfTakenPop(%this, %flagRef) {

	if (!$uberPref::UT::popupCTF) return;

	if (teamTracker.getSelfRef() == %flagRef.actorCurrent) %name = "You";
	else %name = strippedName(%flagRef.actorCurrent);

	if (%flagRef.teamID == teamTracker.friendlyTeamID) {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:ff9999>took your flag from the stand",2);
	}
	else {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:22ff22>took the<spush><color:ff9999>" SPC teamTracker.teamName[%flagRef.teamID] NL
			 "<spop>flag from the stand",3);
	}
}

function uberTracker::ctfDropPop(%this, %flagRef) {

	if (!$uberPref::UT::popupCTF) return;

	if (teamTracker.getSelfRef() == %flagRef.actorCurrent) %name = "You";
	else %name = strippedName(%flagRef.actorCurrent);

	if (%flagRef.teamID == teamTracker.friendlyTeamID) {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:22ff22>dropped your flag in the field",2);
	}
	else {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:ff9999>dropped the<spush><color:ff9999>" SPC teamTracker.teamName[%flagRef.teamID] NL
			 "<spop>flag",3);
	}
}

function uberTracker::ctfCapturePop(%this, %flagRef) {

	if (!$uberPref::UT::popupCTF) return;

	if (teamTracker.getSelfRef() == %flagRef.actorCurrent) %name = "You";
	else %name = "<spush><color:ffffff>" @ strippedName(%flagRef.actorCurrent) @ "<spop>";

	if (%flagRef.teamID == teamTracker.friendlyTeamID) {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:ff9999>captured your flag",2);
	}
	else {
		%this.display("<just:center><font:Univers:16><color:ffffff>" @ %name NL
			"<color:22ff22>captured the<spush><color:ff9999>" SPC teamTracker.teamName[%flagRef.teamID] NL
			"<spop>flag",3);
	}
}

// This function is the main kill tracking engine for the script.
//=============================================================================
function uberTracker::onKill(%this, %type, %killerRef, %victimRef, %weapon, %i_die, %i_win, %suicide, %tk) {

	//warn(%type @ ":" SPC %killerRef.name SPC "killed" SPC %victimRef.name SPC "with the\c6" SPC %weapon @ "." SPC %i_win SPC %i_die SPC %suicide SPC %tk);

	if (%tk && (MissionCallback.getMissionType() $= "Deathmatch")) %tk = false;

	if (isPlayingDemo() || ($FT_gameType $= "SinglePlayerGame")) return;

	if (%i_win || %i_die) {

		if(%suicide) {
			if (%weapon $= "suicide") {
				%this.logData.ctrlK++;
				%this.mapData.ctrlK++;
				return;
			}
			else {
				%this.logData.suicides++;
				%this.mapData.suicides++;
				%this.showSuicidePopup();
			}
			return;
		}

		if (%i_win) {
			if (%victimRef.isBot) %subject = " BOT ";
			else %subject = stripMLControlChars(%victimRef.name);
		}
		else if (%i_die) {
			if (%killerRef.isBot) %subject = " BOT ";
			else %subject = stripMLControlChars(%killerRef.name);
		}

		// Only credit a player if there was a named killer (ignore turrets, falls, etc.)
		if (trim(%subject) !$= "") {

			%logPlayerData = %this.logData.playerData.value(%subject);
			if (!isObject(%logPlayerData)) {
				%logPlayerData = %this.createTrackerInfo(%subject);
				%this.logData.playerData.add(%subject,%logPlayerData);
			}

			%mapPlayerData = %this.mapData.playerData.value(%subject);
			if (!isObject(%mapPlayerData)) {
				%mapPlayerData = %this.createTrackerInfo(%subject);
				%this.mapData.playerData.add(%subject,%mapPlayerData);
			}
		}

		%logWeaponData = %this.logData.weaponData.value(%weapon);
		if (!isObject(%logWeaponData)) {
			%logWeaponData = %this.createTrackerInfo(%weapon);
			%this.logData.weaponData.add(%weapon,%logWeaponData);
		}

		%mapWeaponData = %this.mapData.weaponData.value(%weapon);
		if (!isObject(%mapWeaponData)) {
			%mapWeaponData = %this.createTrackerInfo(%weapon);
			%this.mapData.weaponData.add(%weapon,%mapWeaponData);
		}

		if ($uberPref::UT::mapDataPop) {
			%popWeapon = %mapWeaponData;
			%popPlayer = %mapPlayerData;
		}
		else {
			%popWeapon = %logWeaponData;
			%popPlayer = %logPlayerData;
		}

		if (%i_win && !%i_die) {

			if (%tk) {
				if (!%logPlayerData.TK_kills) %this.logData.TK_victims++;
				if (!%mapPlayerData.TK_kills) %this.mapData.TK_victims++;

				if (%logPlayerData) %logPlayerData.TK_kills++;
				if (%mapPlayerData) %mapPlayerData.TK_kills++;

				// since there's no way to back out bot kills from weapon stats
				// don't count bot kills for them
				if (!%victimRef.isBot) {

					%logWeaponData.TK_kills++;
					%mapWeaponData.TK_kills++;

					%this.logData.TK_kills++;
					%this.mapData.TK_kills++;
				}

				%this.showPopup(%subject,%weapon,%popPlayer,%popWeapon,true,true,%victimRef.isBot);
			}
			else {
				if (!%logPlayerData.kills) %this.logData.victims++;
				if (!%mapPlayerData.kills) %this.mapData.victims++;

				if (%logPlayerData) %logPlayerData.kills++;
				if (%mapPlayerData) %mapPlayerData.kills++;

				if (!%victimRef.isBot) {

					%logWeaponData.kills++;
					%mapWeaponData.kills++;

					%this.logData.kills++;
					%this.mapData.kills++;
				}
				%this.showPopup(%subject,%weapon,%popPlayer,%popWeapon,true,false,%victimRef.isBot);
				%this.updateVictim(%victimRef);	//Update Score HUD
			}
		}
		else if (%i_die) {

			if (%tk) {
				if (!%logPlayerData.TK_deaths) %this.logData.TK_killers++;
				if (!%mapPlayerData.TK_deaths) %this.mapData.TK_killers++;

				if (%logPlayerData) %logPlayerData.TK_deaths++;
				if (%mapPlayerData) %mapPlayerData.TK_deaths++;

				if (!%killerRef.isBot) {

					%logWeaponData.TK_deaths++;
					%mapWeaponData.TK_deaths++;

					%this.logData.TK_deaths++;
					%this.mapData.TK_deaths++;
				}
				%this.showPopup(%subject,%weapon,%popPlayer,%popWeapon,false,true,%killerRef.isBot);
			}
			else {
				if (isObject(%logPlayerData) && !%logPlayerData.deaths) %this.logData.killers++;
				if (isObject(%mapPlayerData) && !%mapPlayerData.deaths) %this.mapData.killers++;

				if (%logPlayerData) %logPlayerData.deaths++;
				if (%mapPlayerData) %mapPlayerData.deaths++;

				if (!%killerRef.isBot) {

					%logWeaponData.deaths++;
					%mapWeaponData.deaths++;

					%this.logData.deaths++;
					%this.mapData.deaths++;
				}
				%this.showPopup(%subject,%weapon,%popPlayer,%popWeapon,false,false,%killerRef.isBot);
			}
		}
		%this.updateScore();
	}
}

function uberTracker::dockScore(%this) {

	if ($uberPref::UT::dockScoreHUD) {
		DockManager.dock(UT_ScoreHUD,"L",ObjectiveHUD,"R");
		DockManager.dock(UT_ScoreHUD,"T",ObjectiveHUD,"T");
	}
	else {
		DockManager.unDock(UT_ScoreHUD,"V");
		DockManager.unDock(UT_ScoreHUD,"H");
	}
}

// Update score HUD kill/death counts
function uberTracker::updateScore(%this) {

	%data = %this.mapData;
	UT_Kills.setText(%data.kills);
	UT_Deaths.setText(%data.deaths + %data.suicides);
}

// Update score HUD victim name
function uberTracker::updateVictim(%this, %victimRef, %count) {

	if (!$uberPref::UT::showScoreHUD) return;

	if (isEventPending(%this.blinkKill)) cancel(%this.blinkKill);

	if (isObject(%victimRef)) {
		%count = 9;
		%name =  stripMLControlChars(%victimRef.name);
		UT_lastKill.setValue(%name);
	}

	if (%count % 2) UT_lastKill.setProfile(UT_OrangeProfile);
	else UT_lastKill.setProfile(UT_WhiteProfile);

	%count--;
	if (%count) %this.blinkKill = %this.schedule(450,updateVictim,"",%count);
	else UT_lastKill.setProfile(UT_GreenProfile);
}

function uberTracker::enqueueEndMsg(%this, %delay) {

	%data = new ScriptObject() {
		text = %this.displayLine;
		delay = %delay;
	};
	%this.endMsgQueue.pushFront(%data);
	if (%this.endMsgQueue.size() == 1) %this.schedule(%delay * 1000,"displayQueuedMsg");
}

function uberTracker::displayQueuedMsg(%this) {

	if (%this.endMsgQueue.size() > 0) {

		%data = %this.endMsgQueue.popBack();
		UT_EndStatsTxt.setText(%data.text);
		%data.delete();

		if (%this.endMsgQueue.size() > 0) {
			%x = 1000 * %this.endMsgQueue.valueAt(%this.endMsgQueue.size() - 1).delay;
			%this.schedule(%x,"displayQueuedMsg");
		}
	}
}

// Show a buffered message on the DebriefGui
function uberTracker::displayEndMsg(%this, %delay) {

	if (%delay <= 0) UT_EndStatsTxt.setText(%this.displayLine);
	else %this.enqueueEndMsg(%delay);
	%this.displayLine = "";
}

// Build up lines in the message buffer
function uberTracker::addLine(%this, %line) {

	%this.displayLine = %this.displayLine @ %line @ "\n";
}

function uberTracker::addFmtLine(%this, %str1, %str2) {

	if (%str2 $= "") %line = "<just:center>" @ %str1;
	else {
		if (%str1 $= "") %line = "<just:right>" @ %str2;
		else %line = "<just:left>" @ %str1 @ "<just:right>" @ %str2;
	}

	%this.displayLine = %this.displayLine @ %line @ "\n";
}

// Clear the message buffer
function uberTracker::clearText(%this) {

	%this.displayLine = "";
}

// Display end-of-map statistics on the DebriefGui
function uberTracker::displayEndStats(%this) {

	Canvas.repaint();
	%sx = getword(canvas.getExtent(), 0) / 640.0;
	%sy = getword(canvas.getExtent(), 1) / 480.0;
	UT_EndStatsDlg.setPosition(%sx * 60,%sy * 240);
	DebriefGui.pushToBack(UT_EndStatsDlg);

	%map = %this.mapData;
	%log = %this.logData;

	%this.clearText();
	// Display overall statistics
	%this.addFmtLine("<color:ffffff>Your Statistics");
	%this.addFmtLine("");
	%this.addFmtLine("<color:06e166>Map Kills<color:ffffff>: <color:d0c07c>" @ %map.kills+0, "<color:06e166>Total Kills<color:ffffff>: <color:d0c07c>" @ %log.kills+0);
	%this.addFmtLine("<color:06e166>Map Deaths<color:ffffff>: <color:d0c07c>" @ %map.deaths+0, "<color:06e166>Total Deaths<color:ffffff>: <color:d0c07c>" @ %log.deaths+0);
	%this.addFmtLine("<color:06e166>Map Suicides<color:ffffff>: <color:d0c07c>" @ %map.suicides+0,"<color:06e166>Total Suicides<color:ffffff>: <color:d0c07c>" @ %log.suicides+0);
	if (objectiveHud.gameType $= "CTFgame") {
		%this.addFmtLine("\n<color:ffffff>CTF Statistics");
		%this.addFmtLine("<color:06e166>Map Captures<color:ffffff>: <color:d0c07c>" @ %map.ctfCapCount+0,"<color:06e166>Total Captures<color:ffffff>: <color:d0c07c>" @ %log.ctfCapCount+0);
		%this.addFmtLine("<color:06e166>Map Assists<color:ffffff>: <color:d0c07c>" @ %map.ctfAssistCount+0,"<color:06e166>Total Assists<color:ffffff>: <color:d0c07c>" @ %log.ctfAssistCount+0);
		%this.addFmtLine("<color:06e166>Map Returns<color:ffffff>: <color:d0c07c>" @ %map.ctfReturnCount+0,"<color:06e166>Total Returns<color:ffffff>: <color:d0c07c>" @ %log.ctfReturnCount+0);
	}

	%this.displayEndMsg();

	// Display weapon statistics
	if (%map.weaponData.size() > 0) {
		%stats = %map.weaponData;
		uberTrackerSort(%stats);
		%this.addFmtLine("<color:06e166>Kills by Weapon<lmargin:0>\n");

		%sz = %stats.size();
		for (%i = 0; %i < %sz; %i++) {
			%key = %stats.keys.valueAt(%i);
			%data = %stats.value(%key);
			if (isObject(%data) && %data.kills > 0) {

				%this.addLine("<just:right><color:06e166><clip%:50>" @ %key @ "<color:ffffff> :</clip><lmargin%:50><just:left><color:d0c07c>" @ %data.kills @ "<lmargin:0>");
			}
		}
		%this.displayEndMsg(5);
	}

	// Display victim / killer list
	%this.addFmtLine("<color:ffffff>Everyone You Killed This Map");
	if (%map.victims == 0) %this.addFmtLine("<color:06e166>You killed no one this map.");
	else {
		%stats = %map.playerData;
		uberTrackerSort(%stats);
		%sz = %stats.size();

		%c[0] = %c[1] = "";
		for (%i = 0; %i < %sz; %i++) {
			%key = %stats.keys.valueAt(%i);
			%data = %stats.value(%key);
			if (isObject(%data)) {
				if (%data.kills > 0) {
					%c[%i % 2] = "<color:06e166>" @ %key @ "<color:ffffff>: <color:d0c07c>" @ %data.kills;
				}
				else {
					%c[%i % 2] = "";
				}
			}
			if (%i % 2) {
				%this.addFmtLine(%c[0],%c[1]);
				%c[0] = %c[1] = "";
			}
		}
		if (%i % 2) %this.addFmtLine(%c[0]);
	}

	%this.addFmtLine("");
	%this.addFmtLine("<color:ffffff>Everyone Who Killed You This Map");
	if (%map.killers == 0) %this.addFmtLine("<color:06e166>No one killed you this map.");
	else {
		uberTrackerSort(%stats,"deathsCompare");
		%sz = %stats.size();

		%c[0] = %c[1] = "";
		for (%i = 0; %i < %sz; %i++) {
			%key = %stats.keys.valueAt(%i);
			%data = %stats.value(%key);
			if (isObject(%data)) {
				if (%data.deaths > 0) {
					%c[%i % 2] = "<color:06e166>" @ %key @ "<color:ffffff>: <color:d0c07c>" @ %data.deaths;
				}
				else {
					%c[%i % 2] = "";
				}
			}
			if (%i % 2) {
				%this.addFmtLine(%c[0],%c[1]);
				%c[0] = %c[1] = "";
			}
		}
		if (%i % 2) %this.addFmtLine(%c[0]);
	}
	%this.displayEndMsg(5);

	if (%this.top10Victims.size() > 0) {
		%this.addFmtLine("<color:ffffff>Your top 10 one-map Victims");
		%sz = %this.top10Victims.size();

		%c[0] = %c[1] = "";
		for (%i = 0; %i < %sz; %i++) {
			%key = %this.top10Victims.keys.valueAt(%i);
			%data = %this.top10Victims.value(%key);
			%c[%i % 2] = "<color:06e166>" @ %key @ "<color:ffffff>: <color:d0c07c>" @ %data;

			//warn(%i SPC (%i % 2) SPC %c[0] SPC %c[1]);
			if (%i % 2) {
				%this.addFmtLine(%c[0],%c[1]);
				%c[0] = %c[1] = "";
			}
		}
		if (%i % 2) %this.addFmtLine(%c[0]);
	}

	%this.addFmtLine("");
	%this.addFmtLine("");

	if (%this.top10Killers.size() > 0) {
		%this.addFmtLine("<color:ffffff>Your top 10 one-map Killers");
		%sz = %this.top10Killers.size();

		%c[0] = %c[1] = "";
		for (%i = 0; %i < %sz; %i++) {
			%key = %this.top10Killers.keys.valueAt(%i);
			%data = %this.top10Killers.value(%key);
			%c[%i % 2] = "<color:06e166>" @ %key @ "<color:ffffff>: <color:d0c07c>" @ %data;

			if (%i % 2) {
				%this.addFmtLine(%c[0],%c[1]);
				%c[0] = %c[1] = "";
			}
		}
		if (%i % 2) %this.addFmtLine(%c[0]);
	}
	%this.displayEndMsg(5);
}

function uberTracker::getMapInfo(%this) {

	%this.startTime = formatTimeString("mmddyy-HHnn");
	%this.mapStart = formatTimeString("mm/dd/yy hh:nn:ss A");

	%this.serverMod   = MissionCallback.getServerMod();
	%this.serverName  = MissionCallback.getServerName();
	%this.missionName = MissionCallback.getMissionName();
	%this.missionType = MissionCallback.getMissionType();
}

// Clear the per-map info now and get a filename timestamp
function uberTracker::onMissionInfo(%this) {

	%this.joinedGame = true;

	// Clean up now old data records
	%sz = %this.mapData.playerData.size();
	for (%i=0; %i < %sz; %i++) {
		%this.mapData.playerData.valueAt(%i).delete();
	}

	%sz = %this.mapData.weaponData.size();
	for (%i=0; %i < %sz; %i++) {
		%this.mapData.weaponData.valueAt(%i).delete();
	}

	// Clean up storage area
	%this.mapData.playerData.clear();
	%this.mapData.weaponData.clear();
	%this.mapData.assetsData.clear();

	%this.mapData.delete();
	%this.mapData = uberTracker_createTrackerMap("map");

	UT_lastKill.setValue("Last Kill");
	UT_Kills.setText(0);
	UT_Deaths.setText(0);

	%this.getMapInfo();
	%this.clearText();
}

// HTML output helper functions
// ============================================================================
function uberTracker::infoRow(%this, %s1, %s2) {

	return "<tr><td class=\"td1\">" @ %s1 @ "</td><td class=\"td2\">" @ %s2 @ "</td></tr>";
}

function uberTracker::dataRow1(%this, %name, %info) {

	return "<tr><td class=\"td3\">" @ %name @ "</td><td class=\"td4\">" @ %info.kills @
		"</td><td class=\"td4\">" @ %info.deaths @ "</td><td class=\"td4\">" @ %info.TK_kills @
		"</td><td class=\"td4\">" @ %info.TK_deaths @ "</td></tr>";
}

function uberTracker::dataRow2(%this, %name, %info) {

	return "<tr><td class=\"td5\">" @ %name @ "</td><td class=\"td6\">" @ %info.kills @
		"</td><td class=\"td6\">" @ %info.deaths @ "</td><td class=\"td6\">" @ %info.TK_kills @
		"</td><td class=\"td6\">" @ %info.TK_deaths @ "</td></tr>";
}

function uberTracker::dataRow3(%this, %name, %info) {

	return "<tr><td class=\"td3\">" @ %name @ "</td><td class=\"td2\">" @ %info @ "</td></tr>";
}

// Export single map HTML
function uberTracker::exportMapData(%this) {

	if (%this.missionName $= "") return;
	if ((%this.mapData.weaponData.size() == 0) && (%this.mapData.assetsData.size() == 0) && (
		(%this.mapData.ctfGrabCount == 0) && (%this.mapData.ctfPickCount == 0))) return;

	uberTrackerSort(%this.mapData.playerData);
	uberTrackerSort(%this.mapData.weaponData);
	uberTrackerSort(%this.mapData.assetsData,"valueCompare");

	if ($PlayingOnline) %name = getField(WONGetAuthInfo(),0);
	else %name = stripTrailingSpaces(strToPlayerName(getField($pref::Player[$pref::Player::Current],0 )));

	%writer = new FileObject();

	%writer.openForWrite("prefs/UberTracker/");
	%writer.close();
	%writer.openForWrite("prefs/UberTracker/MapData/");
	%writer.close();

	//if (!isFile("prefs/UberTracker/MapData/mapdata.css")) {
	//	%this.createCSS("prefs/UberTracker/MapData/mapdata.css");
	//}

	%writer.openForWrite("prefs/UberTracker/MapData/" @
		%this.startTime @ "-" @ %this.missionName @ " (" @ %this.missionType @ ").html");

	%writer.writeLine("<html>\n<head>\n\t<title>" @ %this.missionName @
		 " (" @ %this.missionType @ ") for " @ %name @ " @ " @ %this.mapStart @ "</title>");
	%writer.writeLine("\t<link rel=\"stylesheet\" href=\"mapdata.css\" type=\"text/css\"></head>");

	%writer.writeLine("<body>");
	%writer.writeLine("\t<table>");
	%writer.writeLine("\t\t" @ %this.infoRow("Map:",%this.missionName @ " (" @ %this.missionType @ ")"));
	%writer.writeLine("\t\t" @ %this.infoRow("Server:",%this.serverName));
	%writer.writeLine("\t\t" @ %this.infoRow("Mod:",%this.serverMod));
	%writer.writeLine("\t\t" @ %this.infoRow("Map Start:",%this.mapStart));
	%writer.writeLine("\t\t" @ %this.infoRow("Map End:",formatTimeString("mm/dd/yy hh:nn:ss A")));
	%writer.writeLine("\t</table><hr>");
	%writer.writeLine("");

	%this.exportBots = true;
	%ctf = ($uberPref::UT::exportCTF && (objectiveHud.gameType $= "CTFgame"));
	%this.exportData(%this.mapData, %writer, %ctf);
}

// Export master log report HTML
function uberTracker::exportLogData(%this) {

	uberTrackerSort(%this.logData.playerData);
	uberTrackerSort(%this.logData.weaponData);
	uberTrackerSort(%this.logData.assetsData,"valueCompare");

	if ($PlayingOnline) %name = getField(WONGetAuthInfo(),0);
	else %name = stripTrailingSpaces(strToPlayerName(getField($pref::Player[$pref::Player::Current],0 )));

	%writer = new FileObject();

	%writer.openForWrite("prefs/UberTracker/");
	%writer.close();

	if (!isFile("prefs/UberTracker/mapdata.css")) {
		%this.createCSS("prefs/UberTracker/mapdata.css");
	}

	%writer.openForWrite("prefs/UberTracker/UberTracker.html");

	%writer.writeLine("<html>\n<head>\n\t<title>UberTracker Stats Page for " @ %name @ " @ " @
		formatTimeString("mm/dd/yy") @ "</title>");
	%writer.writeLine("\t<link rel=\"stylesheet\" href=\"mapdata.css\" type=\"text/css\"></head>");

	%writer.writeLine("<body>");
	%writer.writeLine("\t<table>");
	%writer.writeLine("\t\t" @ %this.infoRow("Player:",%name));
	%writer.writeLine("\t\t" @ %this.infoRow("Log Started:",%this.startDate));
	%writer.writeLine("\t\t" @ %this.infoRow("v2 Started:",%this.v2startDate));
	%writer.writeLine("\t\t" @ %this.infoRow("Log Updated:",formatTimeString("mm/dd/yy hh:nn:ss A")));
	%writer.writeLine("\t</table><hr>");
	%writer.writeLine("");

	%this.exportBots = false;
	%this.exportData(%this.logData, %writer, true);
}

// Export HTML common to both report types
function uberTracker::exportData(%this, %trackerData, %writer, %ctf) {

	%writer.writeLine("\t<table>");
	%writer.writeLine("\t\t" @ %this.infoRow("Victims:",%trackerData.victims));
	%writer.writeLine("\t\t" @ %this.infoRow("Killers:",%trackerData.killers));
	%writer.writeLine("\t</table><hr><table>");
	%writer.writeLine("\t\t" @ %this.infoRow("Kills:",%trackerData.kills));
	%writer.writeLine("\t\t" @ %this.infoRow("Deaths:",%trackerData.deaths));
	%writer.writeLine("\t\t" @ %this.infoRow("TKs:",%trackerData.TK_kills));
	%writer.writeLine("\t\t" @ %this.infoRow("TKd:",%trackerData.TK_deaths));
	%writer.writeLine("\t\t" @ %this.infoRow("Suicides:",%trackerData.suicides));
	%writer.writeLine("\t\t" @ %this.infoRow("Ctrl-K's:",%trackerData.ctrlK));
	%writer.writeLine("\t</table><hr>");

	if (%ctf) {

		%writer.writeLine("");
		%writer.writeLine("\t<table>");
		%writer.writeLine("\t\t<tr><th colspan=\"2\">CTF Statistics (v2)</th></tr>");
		if (%trackerData.ctfGrabCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Stand Grabs",%trackerData.ctfGrabCount + 0));
		if (%trackerData.ctfPickCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Field Grabs",%trackerData.ctfPickCount + 0));
		if (%trackerData.ctfCapCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Total Captures",%trackerData.ctfCapCount + 0));
		if (%trackerData.ctfCleanCapCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Unassisted Captures",%trackerData.ctfCleanCapCount + 0));
		if (%trackerData.ctfAssistCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Total Assists",%trackerData.ctfAssistCount + 0));
		if (%trackerData.ctfCapOnAssistCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Assists, Self-Cap",%trackerData.ctfCapOnAssistCount + 0));
		if (%trackerData.ctfAssistOtherCapperCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Assists, Other Cap",%trackerData.ctfAssistOtherCapperCount + 0));
		if (%trackerData.ctfGrabCausedCapCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Assists, Initial Grab",%trackerData.ctfGrabCausedCapCount + 0));
		if (%trackerData.ctfDropCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Deaths while Carrying",%trackerData.ctfDropCount + 0));

		if ((%trackerData.ctfCarrierKillCount > 0)  || (%trackerData.ctfReturnCount > 0))
			%writer.writeLine("\t\t<tr><th colspan=\"2\"><font size=\"-6\">&nbsp;</font></th></tr>");
		if (%trackerData.ctfCarrierKillCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Carrier Kills",%trackerData.ctfCarrierKillCount + 0));
		if (%trackerData.ctfReturnCount)
			%writer.writeLine("\t\t" @ %this.dataRow3("Returns",%trackerData.ctfReturnCount + 0));

		if ((%trackerData.ctfBestGrabSpeed > 0) || %trackerData.ctfBestCapTime > 0)
			%writer.writeLine("\t\t<tr><th colspan=\"2\"><font size=\"-6\">&nbsp;</font></th></tr>");
		if (%trackerData.ctfBestGrabSpeed > 0)
			%writer.writeLine("\t\t" @ %this.dataRow3("Highest Grab Velocity",%trackerData.ctfBestGrabSpeed + 0));
		if (%trackerData.ctfBestCapTime > 0)
			%writer.writeLine("\t\t" @ %this.dataRow3("Shortest Capture Time",%trackerData.ctfBestCapTime/1000 @ "s"));
		%writer.writeLine("\t</table><hr>");
	}

	if (%trackerData.assetsData.size() > 0) {

		%writer.writeLine("");
		%writer.writeLine("\t<table>");
		%writer.writeLine("<tr><th colspan=\"2\">Enemy Assets Destroyed (v2)</th></tr>");

		%sz = %trackerData.assetsData.size();
		for (%i = 0; %i < %sz; %i++) {
			%key = %trackerData.assetsData.keys.valueAt(%i);
			%count = %trackerData.assetsData.value(%key);
			%name = %this.statName[%key];
			if (%name $= "") %name = %key;
			%writer.writeLine("\t\t" @ %this.dataRow3(%name,%count));
		}
		%writer.writeLine("\t</table><hr>");
	}

	if (%trackerData.weaponData.size() > 0) {
		%writer.writeLine("");
		%writer.writeLine("\t<table>");
		%writer.writeLine("<tr><th>Weapon</th><th>Kills With</th><th>Deaths By</th><th>TKs With</th><th>TKs By</th></tr>");

		%sz = %trackerData.weaponData.size();
		for (%i = 0; %i < %sz; %i++) {
			%key = %trackerData.weaponData.keys.valueAt(%i);

			%data = %trackerData.weaponData.value(%key);
			if (isObject(%data)) {
				%writer.writeLine("\t\t" @ %this.dataRow1(%key,%data));
			}
		}
		%writer.writeLine("\t</table><hr>");
	}

	if (%trackerData.playerData.size() > 0) {
		%writer.writeLine("");
		%writer.writeLine("\t<table>");
		%writer.writeLine("<tr><th>Player</th><th>Times Killed</th><th>Killed By</th><th>TKd</th><th>TKd By</th></tr>");

		%sz = %trackerData.playerData.size();
		for (%i = 0; %i < %sz; %i++) {
			%key = %trackerData.playerData.keys.valueAt(%i);

			if ((%key !$= " BOT ") || %this.exportBots) {
				%data = %trackerData.playerData.value(%key);
				if (isObject(%data)) {
					%writer.writeLine("\t\t" @ %this.dataRow2(%key,%data));
				}
			}
		}
		%writer.writeLine("\t</table>");
	}

	%writer.close();
	%writer.delete();
}

// Generate a style sheet
function uberTracker::createCSS(%this, %filename) {

	%writer = new FileObject();
	%writer.openForWrite(%filename);

	%reader = new FileObject();
	%reader.openForRead("scripts/UberGuy/mapdata.css");

	while (!%reader.isEOF()) {
		%writer.writeLine(%reader.readLine());
	}

	%writer.close(); %writer.delete();
	%reader.close(); %reader.delete();
}

// Update list of top-10 killers and victems
function uberTracker::recalcTopTen(%this) {

	%players = %this.mapData.playerData;
	%sz = %players.size();
	for (%i = 0; %i < %sz; %i++) {
		%key = %players.keys.valueAt(%i);
		%data = %players.value(%key);
		if (isObject(%data)) {
			if (%this.top10Victims.value(%key) < %data.kills) %this.top10Victims.add(%key,%data.kills);
			if (%this.top10Killers.value(%key) < %data.deaths) %this.top10Killers.add(%key,%data.deaths);
		}
	}
	%this.top10Victims.remove(" BOT ");
	%this.top10Killers.remove(" BOT ");

	uberTrackerSort(%this.top10Victims,"valueCompare");
	%sz = %this.top10Victims.size();
	while(%sz > 10)
		%this.top10Victims.remove(%this.top10Victims.keys.valueAt(%sz--));

	uberTrackerSort(%this.top10Killers,"valueCompare");
	%sz = %this.top10Killers.size();
	while(%sz > 10)
		%this.top10Killers.remove(%this.top10Killers.keys.valueAt(%sz--));
}

// Game event managers
// ============================================================================
function uberTracker::onGameOver(%this) {

	%this.recalcTopTen();
	%this.saveTrackerInfo();
	if ($uberPref::UT::savePerMap) %this.exportMapData();
}

function uberTracker::onClientDrop(%this) {

	%this.recalcTopTen();
	%this.saveTrackerInfo();
	if ($uberPref::UT::savePerMap) %this.exportMapData();
}

// Export main log file
function uberTracker::saveTrackerInfo(%this) {

	if (isPlayingDemo()) return;

	if (!%this.joinedGame) return;

	%this.joinedGame = false;

	if (isFile("prefs/UberTracker/UberTracker.txt")) {
		if (isFile("prefs/UberTracker/UberTracker.bak.txt"))
			deleteFile("prefs/UberTracker/UberTracker.bak.txt");
		renameFile("prefs/UberTracker/UberTracker.txt","prefs/UberTracker/UberTracker.bak.txt");
	}

	%writer = new FileObject();

	uberTrackerSort(%this.logData.playerData);
	uberTrackerSort(%this.logData.weaponData);

	%writer.openForWrite("prefs/UberTracker/");
	%writer.close();

	%writer.openForWrite("prefs/UberTracker/UberTracker.txt");

	%now = formatTimeString("mm/dd/yy hh:nn:ss A");
	if (%this.startDate $= "") %this.startDate = %now;
	%writer.writeLine("#LogStarted\t" @ %this.startDate);
	if (%this.v2StartDate $= "") %this.v2startDate = %now;
	%writer.writeLine("#LogV2Start\t" @ %this.v2startDate);
	%writer.writeLine("#LogUpdated\t" @ %now);

	%writer.writeLine("");

	%writer.writeLine("#Victims\t" @ %this.logData.victims);
	%writer.writeLine("#Killers\t" @ %this.logData.killers);
	%writer.writeLine("#TK_Victims\t" @ %this.logData.TK_victims);
	%writer.writeLine("#TK_Killers\t" @ %this.logData.TK_killers);

	%writer.writeLine("");

	%writer.writeLine("#Kills\t" @ %this.logData.kills);
	%writer.writeLine("#Deaths\t" @ %this.logData.deaths);
	%writer.writeLine("#I_TKd\t" @ %this.logData.TK_kills);
	%writer.writeLine("#TKd_me\t" @ %this.logData.TK_deaths);
	%writer.writeLine("#Suicides\t" @ %this.logData.suicides);
	%writer.writeLine("#Ctrl-K\t" @ %this.logData.ctrlK);

	%writer.writeLine("");

	%writer.writeLine("#CTF_Stand_Grabs\t" @ %this.logData.ctfGrabCount);
	%writer.writeLine("#CTF_Field_Grabs\t" @ %this.logData.ctfPickCount);
	%writer.writeLine("#CTF_Caps\t" @ %this.logData.ctfCapCount);
	%writer.writeLine("#CTF_Clean_Caps\t" @ %this.logData.ctfCleanCapCount);
	%writer.writeLine("#CTF_Assists\t" @ %this.logData.ctfAssistCount);
	%writer.writeLine("#CTF_Assisting_Caps\t" @ %this.logData.ctfCapOnAssistCount);
	%writer.writeLine("#CTF_Assists_Other\t" @ %this.logData.ctfAssistOtherCapperCount);
	%writer.writeLine("#CTF_Assists_Grabbed\t" @ %this.logData.ctfGrabCausedCapCount);
	%writer.writeLine("#CTF_Returns\t" @ %this.logData.ctfReturnCount);
	%writer.writeLine("#CTF_Carrier_Kills\t" @ %this.logData.ctfCarrierKillCount);
	%writer.writeLine("#CTF_Carrier_Deaths\t" @ %this.logData.ctfDropCount);
	%writer.writeLine("#CTF_Fastest_Grab\t" @ %this.logData.ctfBestGrabSpeed);
	%writer.writeLine("#CTF_Fastest_Cap\t" @ %this.logData.ctfBestCapTime);

	%writer.writeLine("");

	%sz = %this.logData.weaponData.size();
	for (%i = 0; %i < %sz; %i++) {
		%key = %this.logData.weaponData.keys.valueAt(%i);

		%data = %this.logData.weaponData.value(%key);
		if (isObject(%data)) {
			%writer.writeLine("#Weapon" TAB %key TAB %data.kills TAB %data.deaths TAB %data.TK_kills TAB %data.TK_deaths);
		}
	}

	%writer.writeLine("");

	%sz = %this.logData.assetsData.size();
	for (%i = 0; %i < %sz; %i++) {
		%key = %this.logData.assetsData.keys.valueAt(%i);
		%count = %this.logData.assetsData.value(%key);
		%writer.writeLine("#Asset" TAB %key TAB %count);
	}

	%writer.writeLine("");

	if (%this.top10Victims.size()) {
		%sz = %this.top10Victims.size();
		for(%i = 0; %i < %sz; %i++) {
			%key = %this.top10Victims.keys.valueAt(%i);
			%data = %this.top10Victims.value(%key);
			%writer.writeLine("#Top10_Victim" TAB %key TAB %data);
		}
		%writer.writeLine("");
	}

	if (%this.top10Killers.size()) {
		%sz = %this.top10Killers.size();
		for(%i = 0; %i < %sz; %i++) {
			%key = %this.top10Killers.keys.valueAt(%i);
			%data = %this.top10Killers.value(%key);
			%writer.writeLine("#Top10_Killer" TAB %key TAB %data);
		}
		%writer.writeLine("");
	}

	%sz = %this.logData.playerData.size();
	for (%i = 0; %i < %sz; %i++) {
		%key = %this.logData.playerData.keys.valueAt(%i);
		%data = %this.logData.playerData.value(%key);
		if (isObject(%data)) {
			%writer.writeLine("#Player" TAB %key TAB %data.kills TAB %data.deaths TAB %data.TK_kills TAB %data.TK_deaths);
		}
	}

	%writer.close();
	%writer.delete();
}

// Read main log file
function uberTracker::loadTrackerInfo(%this) {

	%this.logData.victims = 0;
	%this.logData.killers = 0;
	%this.logData.TK_victims = 0;
	%this.logData.TK_killers = 0;

	if (!isFile("prefs/UberTracker/UberTracker.txt")) return;

	%reader = new FileObject();

	%reader.openForRead("prefs/UberTracker/UberTracker.txt");

	while (!%reader.isEOF()) {

		%line = trim(%reader.readLine());
		%first = getField(%line,0);
		if (getSubStr(%first,0,1) !$= "#") continue;

		switch$ (%first) {

			case "#Player":
				%player = getField(%line,1);
				%logPlayerData = %this.logData.playerData.value(%player);
				if (!isObject(%logPlayerData)) {
					%logPlayerData = %this.createTrackerInfo(%player);
					%this.logData.playerData.add(%player,%logPlayerData);
				}
				%logPlayerData.kills = getField(%line,2);
				%logPlayerData.deaths = getField(%line,3);
				%logPlayerData.TK_kills = getField(%line,4);
				%logPlayerData.TK_deaths = getField(%line,5);

				if (%logPlayerData.kills)     %this.logData.victims++;
				if (%logPlayerData.deaths)    %this.logData.killers++;
				if (%logPlayerData.TK_kills)  %this.logData.TK_victims++;
				if (%logPlayerData.TK_deaths) %this.logData.TK_killers++;

			case "#Weapon":
				%weapon = getField(%line,1);
				%logWeaponData = %this.logData.weaponData.value(%weapon);
				if (!isObject(%logWeaponData)) {
					%logWeaponData = %this.createTrackerInfo(%weapon);
					%this.logData.weaponData.add(%weapon,%logWeaponData);
				}
				%logWeaponData.kills = getField(%line,2);
				%logWeaponData.deaths = getField(%line,3);
				%logWeaponData.TK_kills = getField(%line,4);
				%logWeaponData.TK_deaths = getField(%line,5);

			case "#Top10_Killer":
				%this.top10Killers.add(getField(%line,1),getField(%line,2));

			case "#Top10_Victim":
				%this.top10Victims.add(getField(%line,1),getField(%line,2));

			case "#Asset":
				%assetsData = %this.logData.assetsData;
				%assetsData.add(getField(%line,1),getField(%line,2));

			case "#LogStarted":
				%this.startDate = getField(%line,1);

			case "#LogV2Start":
				%this.v2startDate = getField(%line,1);

			case "#Kills":
				%this.logData.kills = getField(%line,1);

			case "#Deaths":
				%this.logData.deaths = getField(%line,1);

			case "#I_TKd":
				%this.logData.TK_kills = getField(%line,1);

			case "#TKd_Me":
				%this.logData.TK_deaths = getField(%line,1);

			case "#Suicides":
				%this.logData.suicides = getField(%line,1);

			case "#Ctrl-K":
				%this.logData.ctrlK = getField(%line,1);

			case "#CTF_Stand_Grabs":
				%this.logData.ctfGrabCount = getField(%line,1);

			case "#CTF_Field_Grabs":
				%this.logData.ctfPickCount = getField(%line,1);

			case "#CTF_Caps":
				%this.logData.ctfCapCount = getField(%line,1);

			case "#CTF_Clean_Caps":
				%this.logData.ctfCleanCapCount = getField(%line,1);

			case "#CTF_Assists":
				%this.logData.ctfAssistCount = getField(%line,1);

			case "#CTF_Assisting_Caps":
				%this.logData.ctfCapOnAssistCount = getField(%line,1);

			case "#CTF_Assists_Other":
				%this.logData.ctfAssistOtherCapperCount = getField(%line,1);

			case "#CTF_Assists_Grabbed":
				%this.logData.ctfGrabCausedCapCount = getField(%line,1);

			case "#CTF_Returns":
				%this.logData.ctfReturnCount = getField(%line,1);

			case "#CTF_Carrier_Kills":
				%this.logData.ctfCarrierKillCount = getField(%line,1);

			case "#CTF_Carrier_Deaths":
				%this.logData.ctfDropCount = getField(%line,1);

			case "#CTF_Fastest_Cap":
				%this.logData.ctfBestCapTime = getField(%line,1);

			case "#CTF_Fastest_Grab":
				%this.logData.ctfBestGrabSpeed = getField(%line,1);
		}
	}
}

// Prefs related stuff

function uberTracker::loadPrefs() {

	if (!$uberPref::loaded) {
		exec("prefs/uberPrefs.cs");
		$uberPref::loaded = 1;
	}

	if ($uberPref::UT::popupCTF $= "") $uberPref::UT::popupCTF = true;
	if ($uberPref::UT::stickyHUD $= "") $uberPref::UT::stickyHUD = true;
	if ($uberPref::UT::showPopup $= "") $uberPref::UT::showPopup = true;
	if ($uberPref::UT::usePowPop $= "") $uberPref::UT::usePowPop = false;
	if ($uberPref::UT::savePerMap $= "") $uberPref::UT::savePerMap = true;
	if ($uberPref::UT::exportHTML $= "") $uberPref::UT::exportHTML = true;
	if ($uberPref::UT::exportCTF $= "") $uberPref::UT::exportCTF = true;
	if ($uberPref::UT::mapDataPop $= "") $uberPref::UT::mapDataPop = true;
	if ($uberPref::UT::popupAssets $= "") $uberPref::UT::popupAssets = true;
	if ($uberPref::UT::xparentPopup $= "") $uberPref::UT::xparentPopup = true;
	if ($uberPref::UT::showScoreHUD $= "") $uberPref::UT::showScoreHUD = true;
	if ($uberPref::UT::dockScoreHUD $= "") $uberPref::UT::dockScoreHUD = false;
	if ($uberPref::UT::xparentScoreHUD $= "") $uberPref::UT::xparentScoreHUD = false;
	if ($uberPref::UT::popupTime $= "") $uberPref::UT::popupTime = 3500;
}

function uberTracker::savePrefs(%this) {

	%this.saveTrackerInfo();
	export("$uberPref::*", "prefs/uberPrefs.cs", false);
	if ($uberPref::UT::exportHTML) %this.exportLogData();
}

// Gui related stuff

function uberTracker::createHuds() {

	if (isObject(uberTrackerPopup)) {
		%ext = uberTrackerPopup.extent;
		%pos = uberTrackerPopup.position;
		uberTrackerPopup.delete();
	}
	if (isObject(uberTrackerPopupTxt)) uberTrackerPopupTxt.delete();

	new ShellFieldCtrl(uberTrackerPopup) {
		profile = $uberPref::UT::xparentPopup ? "GuiDefaultProfile" : "GuiChatBackProfile";
		horizSizing = "right";
		vertSizing = "bottom";
		position = ((%pos $= "") ? "271 393" : %pos);
		extent = ((%ext $= "") ? "170 50" : %ext);
		minExtent = "16 18";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";

		new GuiMLTextCtrl(uberTrackerPopupTxt) {
			profile = "GuiAmmoHudProfile";
			horizSizing = "width";
			vertSizing = "height";
			position = "0 00";
			extent = "170 50";
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			lineSpacing = "2";
			allowColorChars = "0";
			maxChars = "-1";
			deniedSound = "InputDeniedSound";
		};
	};
	playGui.add(uberTrackerPopup);
	uberTrackerPopup.setVisible(false);

	//Score HUD

	if (isObject(UT_ScoreHUD)) {

		while (UT_ScoreHUD.getCount() > 0) {
			UT_ScoreHUD.getObject(0).delete();
		}
		UT_ScoreHUD.delete();
	}
	else {
		new GuiControlProfile(UT_WhiteProfile)
		{
		    fontType = "Univers Condensed";
		    fontSize = 16;
		    fontColor = "230 230 255";
		    justify = "left";
		};
		new GuiControlProfile(UT_GreenProfile)
		{
		    fontType = "Univers Condensed";
		    fontSize = 16;
		    fontColor = "118 222 48";
		    justify = "left";
		};
		new GuiControlProfile(UT_OrangeProfile)
		{
		    fontType = "Univers Condensed";
		    fontSize = 16;
		    fontColor = "250 175 0";
		    justify = "left";
		};
		new GuiControlProfile(UT_RedProfile)
		{
		    fontType = "Univers Condensed";
		    fontSize = 16;
		    fontColor = "250 20 20";
		    justify = "left";
		};
	}

	new GuiControl(UT_ScoreHUD) {
		profile = "GuiDefaultProfile";
		horizSizing = "right";
		vertSizing = "bottom";
		position = "367 731";
		extent = "110 37";
		minExtent = "8 8";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";

		new HudScoreCtrl() {
			profile = "HudScoreProfile";
			horizSizing = "right";
			vertSizing = "top";
			position = "0 0";
			extent = "110 37";
			minExtent = "16 37";
			visible = !$uberPref::UT::xparentScoreHUD;
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			fillColor = "0.250000 0.250000 0.250000 0.250000";
			frameColor = "0.000000 1.000000 0.000000 1.000000";
			opacity = "1";
			verticalInset = "3 0";
			horizontalInset = "2 2";
		};
		new GuiTextCtrl(UT_KillText) {
			profile = "UT_WhiteProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "4 19";
			extent = "20 16";
			visible = "1";
			text = "Kills:";
		};
		new GuiTextCtrl(UT_DeathText) {
			profile = "UT_WhiteProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "50 19";
			extent = "33 16";
			visible = "1";
			text = "Deaths:";
		};
		new GuiTextCtrl(UT_Kills) {
			profile = "UT_GreenProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "30 19";
			extent = "40 16";
			visible = "1";
			text = "0";
		};
		new GuiTextCtrl(UT_Deaths) {
			profile = "UT_GreenProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "95 19";
			extent = "33 16";
			visible = "1";
			text = "0";
		};
		new GuiTextCtrl(UT_LastKill) {
			profile = "UT_GreenProfile";
			horizSizing = "left";
			vertSizing = "bottom";
			position = "4 2";
			extent = "110 16";
			visible = "1";
			text = "Last Kill";
		};
	};
	playGui.add(UT_ScoreHUD);
	UT_ScoreHUD.setVisible($uberPref::UT::showScoreHUD);

	if (isObject(UT_EndStatsDlg)) {
		UT_EndStatsTxt.delete();
		UT_EndStatsDlg.delete();
	}
	else {
		new GuiControlProfile ("UT_EndStatsTxtProfile")
		{
			fontType = "Univers";
			fontSize = 16;
			fontColor = "169 215 250";
			fontColors[6] = $PlayerNameColor;
			fontColors[7] = $TribeTagColor;
			fontColors[8] = $SmurfNameColor;
			fontColors[9] = $BotNameColor;
		};
	}

	new ShellFieldCtrl(UT_EndStatsDlg) {
		profile = "GuiDefaultProfile";
		horizSizing = "center";
		vertSizing = "top";
		position = "60 240";
		extent = "320 240";
		minExtent = "8 8";
		visible = "1";
		helpTag = "0";

		new GuiMLTextCtrl(UT_EndStatsTxt) {
			profile = "UT_EndStatsTxtProfile";
			horizSizing = "center";
			vertSizing = "top";
			position = "0 0";
			extent = "320 240";
			minExtent = "8 8";
			visible = "1";
			helpTag = "0";
			lineSpacing = "0";
		};
	};
	DebriefGui.add(UT_EndStatsDlg);
}

// Display text in either built-in popup or PowPop
function uberTracker::display(%this, %text, %lines) {

	if (isEventPending(%this.popupID)) cancel(%this.popupID);

	if ($uberPref::UT::stickyHUD && !$uberPref::UT::usePowPop) {
		%xpos = mFloor(getWord($pref::video::resolution, 0) / 2) - mFloor(getWord(uberTrackerPopup.extent, 0) / 2);
		%ypos = getWord($pref::video::resolution, 1) * 0.65;
		if (isObject(PowPopHud)) %ypos += 60;

		uberTrackerPopup.setPosition(%xpos,%ypos);
	}

	if ($uberPref::UT::usePowPop && isObject(PowPop)) {
		PowPop.schedule(PowPop.queue(), display, %text, %lines);
	}
	else {
		uberTrackerPopupTxt.setValue(%text);
		uberTrackerPopup.setExtent(firstWord(uberTrackerPopup.extent,0),%lines * 16.666);
		uberTrackerPopup.setVisible(true);
		%this.popupID = %this.schedule($uberPref::UT::popupTime,"hidePopup");
	}
}

// Specialized popup for suicides
function uberTracker::showSuicidePopup(%this) {

	if (!$uberPref::UT::showPopup) {
		uberTracker.hidePopup();
		return;
	}

	%this.display("<just:center><color:ff9999><spush><font:Univers Bold:16>You killed YOURSELF!<spop>", 1);
}

// Display built-in popup
function uberTracker::showPopup(%this, %player, %weapon, %pData, %wData, %i_win, %tk, %isBot) {

	if (!$uberPref::UT::showPopup) {
		uberTracker.hidePopup();
		return;
	}

	if (%i_win) {
		if (%tk) %action = "<color:ff9999><spush><font:Univers Bold:16>You TEAMKILLED!<spop>";
		else %action = "<color:22ff22><spush><font:Univers Bold:16>You scored a Kill!<spop>";
	}
	else {
		if (%tk) %action = "<color:ff9999><spush><font:Univers Bold:16>You were TEAMKILLED!<spop>";
		else %action = "<color:ff9999><spush><font:Univers Bold:16>You died!<spop>";
	}
	%lines++;

	if (isObject(%pData)) {
		%subject = "<color:ffffff>" @ %player @ "<color:ffff00> (" @
			(%tk ? %pData.TK_kills : %pData.kills) @ "/" @ (%tk ? %pData.TK_deaths : %pData.deaths) @ ")\n";
		%lines++;
	}

	if (isObject(%wData) && !%isBot) {
		%implement = "<color:ffffff>" @ %weapon;
		if (%subject !$= "") {
			%implement = %implement @ "<color:ffff00> (" @
				(%tk ? %wData.TK_kills : %wData.kills) @ "/" @
				(%tk ? %wData.TK_deaths : %wData.deaths) @ ")";
		}
		else %implement = "<color:22ff22>Killed By: " @ %implement;
		%lines++;
	}

	%this.display("<just:center>" @ %action NL %subject @ %implement, %lines);
}

function uberTracker::hidePopup(%this) {

	if (isEventPending(%this.popupID)) cancel(%this.popupID);
	uberTrackerPopup.setVisible(false);
}

function UberTrackerGUI::onWake(%this) {

	//nada
}

if (!isObject(UberTrackerGUI)) {

	new ShellFieldCtrl(UberTrackerGUI) {
		profile = "GuiDefaultProfile";
		horizSizing = "width";
		vertSizing = "height";
		position = "0 0";
		extent = "400 350";
		minExtent = "16 18";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";
	};
}

while (UberTrackerGUI.getCount() > 0) {
	UberTrackerGUI.getObject(0).delete();
}

new ShellFieldCtrl(UberTrackerGUI2) {
	profile = "ShellFieldProfile";
	horizSizing = "center";
	vertSizing = "center";
	position = "43 31";
	extent = "314 288";
	minExtent = "16 18";
	visible = "1";
	hideCursor = "0";
	bypassHideCursor = "0";
	helpTag = "0";

	new ShellPaneCtrl() {
		profile = "ShellPaneProfile";
		horizSizing = "right";
		vertSizing = "bottom";
		position = "4 15";
		extent = "306 268";
		minExtent = "48 92";
		visible = "1";
		hideCursor = "0";
		bypassHideCursor = "0";
		helpTag = "0";
		text = "UberTracker";
		longTextBuffer = "0";
		maxLength = "255";
		noTitleBar = "0";

		new GuiTextCtrl() {
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "37 164";
			extent = "60 17";
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			text = "PopUp Time";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellTextEditCtrl(spawnMuteTime) {
			profile = "NewTextEditProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "92 156";
			extent = "68 39";
			minExtent = "32 38";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::popupTime";
			helpTag = "0";
			longTextBuffer = "0";
			maxLength = "4";
			historySize = "0";
			password = "0";
			IRCName = "0";
			tabComplete = "0";
			deniedSound = "InputDeniedSound";
			glowOffset = "9 9";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 134";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::mapDataPop";
			helpTag = "0";
			text = "Popup Map Stats";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 159";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::popupCTF";
			helpTag = "0";
			text = "Popup CTF Events";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 134";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::popupAssets";
			helpTag = "0";
			text = "Popup Asset Kills";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 59";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::usePowPop";
			helpTag = "0";
			text = "Use PowPop Popup";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 34";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::exportCTF";
			helpTag = "0";
			text = "Export CTF Stats";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 189";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::savePerMap";
			helpTag = "0";
			text = "Save Map Stats";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 214";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::exportHTML";
			helpTag = "0";
			text = "Save HTML on quit";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 34";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::showPopup";
			helpTag = "0";
			text = "Show Popup";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 59";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::showScoreHUD";
			command = "UT_scoreHUD.setVisible(!UT_scoreHUD.visible);";
			helpTag = "0";
			text = "Show Kills HUD";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 84";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::xparentPopup";
			command = "uberTracker.createHuds();";
			helpTag = "0";
			text = "Transparent Popup";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "32 109";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::stickyHUD";
			helpTag = "0";
			text = "\"Sticky\" Popup";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 84";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::xparentScoreHUD";
			command = "uberTracker.createHuds();";
			helpTag = "0";
			text = "Transparent HUD";
			longTextBuffer = "0";
			maxLength = "255";
		};
		new ShellToggleButton() {
			profile = "ShellRadioProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "152 109";
			extent = "115 30";
			minExtent = "26 27";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			variable = "$uberPref::UT::dockScoreHUD";
			command = "uberTracker.dockScore();";
			helpTag = "0";
			text = "Dock Kills HUD";
			longTextBuffer = "0";
			maxLength = "255";
		};
	};
};

if (!isObject(uberTracker)) {

	new ScriptObject(uberTracker) {
		class = "uberTracker";

		mapData = uberTracker_createTrackerMap("map");
		logData = uberTracker_createTrackerMap("log");

		top10Killers = Container::newVectorMap();
		top10Victims = Container::newVectorMap();

		endMsgQueue = Container::newList();
	};
}

// Message callbacks and lookup tables for asset destruction events

uberTracker.statType["msgVSDes"] = "VPad";
uberTracker.statType["msgGenDes"] = "Gen";
uberTracker.statType["msgInvDes"] = "Invo";
uberTracker.statType["msgSolarDes"] = "Solar";
uberTracker.statType["msgDepInvDes"] = "DepInvo";
uberTracker.statType["msgDepTurDes"] = "DepTurret";
uberTracker.statType["msgSensorDes"] = "Sensor";
uberTracker.statType["msgSentryDes"] = "Sentry";
uberTracker.statType["msgTurretDes"] = "Turret";
uberTracker.statType["msgDepSensorDes"] = "DepSensor";
uberTracker.statType["msgVehicleScore"] = "Vehicle";

uberTracker.statName["VPad"] = "Vehicle Pad";
uberTracker.statName["Gen"] = "Generator";
uberTracker.statName["Invo"] = "Inventory";
uberTracker.statName["Solar"] = "Solar Panel";
uberTracker.statName["DepInvo"] = "Deployed Inventory";
uberTracker.statName["DepTurret"] = "Deployed Turret";
uberTracker.statName["Sensor"] = "Sensor";
uberTracker.statName["Sentry"] = "Sentry Turret";
uberTracker.statName["Turret"] = "Turret";
uberTracker.statName["DepSensor"] = "Deployed Sensor";

uberTracker.statName["MPB"] = "MPB";
uberTracker.statName["Bomber"] = "Bomber";
uberTracker.statName["Turbograv"] = "Shrike";
uberTracker.statName["Grav Cycle"] = "Grav Cycle";
uberTracker.statName["Heavy Transport"] = "Havoc";
uberTracker.statName["Assault Tank"] = "Assault Tank";

addMessageCallback('msgVSDes',        uberTracker_handleDestroy);
addMessageCallback('msgGenDes',       uberTracker_handleDestroy);
addMessageCallback('msgInvDes',       uberTracker_handleDestroy);
addMessageCallback('msgSolarDes',     uberTracker_handleDestroy);
addMessageCallback('msgDepInvDes',    uberTracker_handleDestroy);
addMessageCallback('msgDepTurDes',    uberTracker_handleDestroy);
addMessageCallback('msgSensorDes',    uberTracker_handleDestroy);
addMessageCallback('msgSentryDes',    uberTracker_handleDestroy);
addMessageCallback('msgTurretDes',    uberTracker_handleDestroy);
addMessageCallback('msgDepSensorDes', uberTracker_handleDestroy);
addMessageCallback('msgVehicleScore', uberTracker_handleDestroy);

// General callbacks

Callback.add(KillCallback,"uberTracker.onKill");

Callback.add(onQuit,"uberTracker.savePrefs");
Callback.add(onGameOver,"uberTracker.onGameOver");
Callback.add(onUserClientDrop,"uberTracker.onGameOver");
Callback.add(onMissionDropInfo,"uberTracker.onMissionInfo");
callback.add(onDebriefGuiWake,"uberTracker.displayEndStats");

Callback.add(onCTFGrab,   "uberTracker.onCTFGrab");
Callback.add(onCTFCap,    "uberTracker.onCTFCap");
Callback.add(onCTFPicked, "uberTracker.onCTFPicked");
Callback.add(onCTFReturn, "uberTracker.onCTFReturn");
Callback.add(CarrierKillCTF, "uberTracker.onCTFKill");

Callback.add(onCTFGrab,   "uberTracker.ctfTakenPop");
Callback.add(onCTFDrop,   "uberTracker.ctfDropPop");
Callback.add(onCTFCap,    "uberTracker.ctfCapturePop");
Callback.add(onCTFPicked, "uberTracker.ctfPickupPop");
Callback.add(onCTFReturn, "uberTracker.ctfReturnPop");

if (isObject(HM) && isObject(HudMover)) {
	HudMover::addHud(uberTrackerPopup,"UberTracker Popup");
	HudMover::addHud(UT_ScoreHUD,"UberTracker Score");
}

UberTrackerGUI.add(UberTrackerGUI2);

uberTracker.loadPrefs();
uberTracker.createHuds();
uberTracker.dockScore();
uberTracker.loadTrackerInfo();